import sys, os, json
import aws_s3_utils

### supersedes aws_s3_utils and other file utils scripts. ###

def writeJSON( myjson, fout_name ):
    """ Writes (dumps) a JSON as a string to a file.
    """
    with open(fout_name,'w') as fout:
        json.dump(myjson, fout)
    return fout_name

def getJSON( fname ):
    return loadJSON(fname)

def loadJSON( fname ):
    """ Loads JSON from file named 'fname' into a JSON object and return this object.

    >>> loadJSON( "foo.json" )
    JSON ERROR - JSON NOT FORMATTED CORRECTLY OR FILE NOT FOUND: [Errno 2] No such file or directory: 'foo.json'
    {}
    """
    try:
        if type(fname) == type([]):
            fname = fname[0] if fname != [] else ''
        with open(fname,'r') as f:
            myjson = json.load(f)
    except Exception as e:
        print('JSON ERROR - JSON NOT FORMATTED CORRECTLY OR FILE NOT FOUND: '+str(fname))
        return {}
    return myjson

def save_files( args ):
    """ Saves local files to a cloud storage provider.

    files: list of files
    path: cloud storage path
    """
    files = args['files']
    cloudpath = args['path']

    if cloudpath.startswith('s3://'):
        for f in files:
            aws_s3_utils.upload_file_s3( dict(localfile=f, path=cloudpath))
    return



import sys, os, yaml, json, subprocess
from pathlib import Path
# from decouple import config # pip install python-decouple
import aws_batch_utils
import quick_utils
import file_utils

SCRIPT_DIR = str(os.path.dirname(os.path.realpath(__file__)))
PKG = yaml.safe_load(Path(os.path.join(SCRIPT_DIR, 'packages.yaml')).read_text())
HOME_PATH = os.path.expanduser('~')
AWS_CONFIG = quick_utils.loadJSON(os.path.join(HOME_PATH,'.bioshedinit/','aws_config_constants.json')) \
             if os.path.exists(os.path.join(HOME_PATH,'.bioshedinit/','aws_config_constants.json')) \
             else quick_utils.loadJSON(os.path.join(SCRIPT_DIR,'aws_config_constants.json'))
CONTAINER_REGISTRY_URL = AWS_CONFIG['ecr_registry']
UTILS = ['program_utils.py', 'file_utils.py', 'aws_s3_utils.py', 'quick_utils.py', 'aws_config_constants.json', 'specs.json']

def run_container_local( args ):
    """ Runs a container on the local system (docker run)
    name: name of container
    tag: (optional) tag of container image
    args: program arguments
    need_sudo: (optional)
    dockerargs: arguments to docker run (optional)
    ---
    status: status code

    [TODO] try except

    >>> run_container_local( dict(name='test'))
    'sudo docker run test:latest '

    """
    # sudo docker run test:latest
    cname = args['name']
    ctag = args['tag'] if 'tag' in args else 'latest'
    pargs = quick_utils.format_type(args['args'], 'string_space') if 'args' in args else ''
    need_sudo = args['need_sudo'] if 'need_sudo' in args else 'y'
    dockerargs = str(args['dockerargs']).strip()+' ' if 'dockerargs' in args else ''
    container_registry = str(CONTAINER_REGISTRY_URL).rstrip('/')

    pullcmd = f'docker pull {container_registry}/{cname}:{ctag}'
    pullcmd = 'sudo '+pullcmd if need_sudo.lower()[0]=='y' else pullcmd
    subprocess.call(pullcmd, shell=True)

    cmd = f'docker run {dockerargs}{container_registry}/{cname}:{ctag} {pargs}'
    cmd = 'sudo '+cmd if need_sudo.lower()[0]=='y' else cmd
    subprocess.call(cmd, shell=True)
    return cmd

def run_container_remote( args ):
    """ Runs a container in a remote system
    name: name of container
    args: program arguments
    system: (optional) which system to run in (awsbatch, kubernetes,...)
    ---
    status: status code
    """
    cname = args['name']
    pargs = args['args'] if 'args' in args else ''
    system = args['system'] if 'system' in args else 'awsbatch'
    job_info = {}

    if system=='awsbatch':
        job_info = aws_batch_utils.submit_job_awsbatch( dict(name=cname, program_args=pargs))
    return job_info

def create_dockerfile( args ):
    """ Creates a Dockerfile for a container
    [TODO] create a global dict of {library: install command }
    [TODO] create a global dict of {dockerbase: dockerbaseimage }
    [TODO] update to adjust install command based on OS (apt-get for Ubuntu, yum for Red Hat)
    [NOTE] run_main.py - ENTRYPOINT for python and command-line programs
    [NOTE] support for simple installs from package repos. Custom installs of downloaded programs to come later.
    name: desired name of container
    requirements: path to package/lib/command requirements file
    base: (optional) base to use for Dockerfile - python2, python3, R, ubuntu, conda (default)
    ---
    dockerfile: path to dockerfile

    >>> create_dockerfile( dict(name="test", requirements="test/requirements.test.txt", base="conda"))
    'test.Dockerfile'

    """
    cname = args['name']
    req_file = args['requirements']
    image_base = args['base'] if 'base' in args else 'conda'
    reqs = []
    cmds = []

    # get package install definitions
    with open(req_file,'r') as f:
        for r in f:
            r = r.strip()
            if r in PKG['ubuntu']:
                # e.g., bedtools
                reqs.append(PKG['ubuntu'][r])
            elif r in PKG['R']:
                reqs.append(PKG['R'][r])
            else:
                # direct command - e.g., conda install -c anaconda bedtools
                # e.g., wget http://myfiles.tar.gz
                cmds.append(r)

    args['dockerfile'] = f'{cname}.Dockerfile'
    args['packages'] = reqs
    args['utils'] = UTILS
    args['commands'] =  cmds
    if image_base in ['python', 'conda', 'ubuntu', 'linux']:
        write_dockerfile_conda( args )
    elif image_base.upper()=='R':
        write_dockerfile_R( args )
    return args['dockerfile']

def create_makefile( args ):
    """ Creates makefile for building container
    name: desired name of container
    dockerfile: (optional) name of dockerfile
    tag: (optional) tag for container image
    ---
    makefile: path to makefile

    [TODO] push to Dockerhub instead. Won't have lowercase limitations

    >>> create_makefile( dict(name="test", dockerfile="test.Dockerfile") )
    'Makefile'
    """
    cname = args['name'].lower()  # ECR only takes lower-case
    dockerfile = args['dockerfile'] if 'dockerfile' in args else 'Dockerfile'
    ctag = args['tag'] if 'tag' in args else 'latest'

    with open('Makefile','w') as f:
        f.write(f'NAME={cname}\n')
        f.write(f'TAG={ctag}\n\n')
        f.write('all: build push\n\n')
        f.write('build:\n')
        f.write(f'\tdocker build -t $(NAME):$(TAG) -f {dockerfile} ./\n')
        f.write('\tdocker tag $(NAME):$(TAG) $(REGISTRY):$(TAG)\n\n')
        f.write('push:\n')
        f.write('\tdocker push $(REGISTRY):$(TAG)\n')
    return 'Makefile'

def run_makefile( args ):
    """ Runs makefile to build docker and create container image
    name: name of container
    url: (optional) url to main container repository to write to
    need_sudo: (optional) default yes

    [TODO] try except for subprocess
    [TODO] push to docker hub instead. Won't have lower case limitations

    >>> run_makefile( dict(name='test'))
    0
    """
    cname = args['name'].lower()
    creg_url = args['url'] if 'url' in args else CONTAINER_REGISTRY_URL
    need_sudo = args['need_sudo'] if 'need_sudo' in args else 'yes'

    cmd = f'make REGISTRY={creg_url}/{cname}'
    cmd = 'sudo '+cmd if need_sudo.lower()[0]=='y' else cmd
    subprocess.call(cmd, shell=True)
    return 0

def build_container( args ):
    """ Builds a new custom container.
    name: container name
    requirements: file listing packages and commands needed to run container
    codebase: codebase needed to run program - R, python, etc...
    ---
    status: status (integer)
    """
    cname = args['name']
    reqfile = args['requirements']
    codebase = args['codebase'] if 'codebase' in args else 'conda'

    dockerfile = create_dockerfile( dict(name=cname, requirements=reqfile, base=codebase))
    makefile = create_makefile( dict(name=cname, dockerfile=dockerfile))
    file_utils.save_files( dict(files=[dockerfile, reqfile], path='s3://hubmodules/{}/'.format(cname)))
    makestatus = run_makefile( dict(name=cname))
    return dict(status=makestatus)

def build_container_cli( cname, reqfile ):
    """ Command-line alias for build_container. Takes regular command-line args.
    cname: container name
    reqfile: requirements file
    """
    return build_container( dict(name=cname, requirements=reqfile))

def create_container( cname, reqfile ):
    """ Creates container - alias for build_container_cli
    """
    return build_container_cli( cname, reqfile )

def translate_command_to_docker( args ):
    """ Translates a command to proper dockerfile language

    name: container name
    command: cmd
    ---
    docker_commands: translated commands for use in dockerfile (as a list)
    """
    BUILT_IN_COMMANDS = ['FROM','ENV','RUN','WORKDIR','ADD','COPY','LABEL','CMD']
    args_out = {}
    docker_commands = []
    cmd = str(args['command']).strip()
    cname = args['name'] if 'name' in args else ''
    # if we already have a docker command, then we add as-is
    if cmd.split(' ')[0] in BUILT_IN_COMMANDS:
        docker_commands.append(cmd)
    elif cmd.startswith('wget'):
        docker_commands.append('RUN '+cmd)
        # get the file name only
        docker_commands.append('RUN tar -xzvf {} -C /'.format(cmd.split(' ')[-1].split('/')[-1]))
    elif cmd.endswith('.tar.gz'):
        # unzip and add to path
        docker_commands.append('RUN mkdir -p /software')
        docker_commands.append('COPY {} /software'.format(cmd))
        docker_commands.append('RUN tar -xzvf /software/{} -C /software'.format(cmd))
        docker_commands.append('ENV PATH="/software/{}/:$PATH"'.format(cmd[:-7]))
        docker_commands.append('ENV PATH="/software/{}/bin/:$PATH"'.format(cmd[:-7]))
    elif cmd.endswith('.zip'):
        # unzip and add to path
        docker_commands.append('COPY {} /'.format(cmd))
        docker_commands.append('RUN unzip /{}'.format(cmd))
        docker_commands.append('ENV PATH="/{}/:$PATH"'.format(cmd[:-7]))
        docker_commands.append('ENV PATH="/{}/bin/:$PATH"'.format(cmd[:-7]))
    elif cmd.startswith('git clone'):
        docker_commands.append('RUN conda install -c anaconda git')
        docker_commands.append('RUN {}'.format(cmd))
        docker_commands.append('ENV PATH="/{}/:$PATH"'.format(cmd.split('/')[-1][:-4]))
    elif cmd.startswith('PATH='):
        # for some programs, need to explicitly add the path
        docker_commands.append('ENV {}:$PATH"'.format(cmd.rstrip('"').rstrip("'")))
    elif cmd.startswith('install.packages('):
        # install R packages
        if '"' in cmd:
            docker_commands.append("RUN R -e '{}'".format(cmd))
        else:
            docker_commands.append('RUN R -e "{}"'.format(cmd))
    elif ('.' in cmd or cmd==cname) and len(cmd.split(' ')) <= 1:
        # assume that we are copying a file over
        docker_commands.append('COPY {} /'.format(cmd))
        docker_commands.append('ENV PATH="/:$PATH"')
    else:
        # otherwise assume we are trying to run some unix command
        docker_commands.append('RUN {}'.format(cmd))
    args_out['docker_commands'] = docker_commands
    return args_out

###########################################################

def write_dockerfile_conda( args ):
    """
    dockerfile: name of dockerfile
    ---
    dockerfile: name of dockerfile
    """
    dockerfile_name = args["dockerfile"]
    container_name = args["name"]
    pkg2install = args['packages']
    cmd2run = args['commands']
    utils_files = args['utils']

    with open(dockerfile_name,'w') as f:
        f.write("FROM continuumio/miniconda3\n")
        f.write('LABEL container.base.image = "miniconda3:python3.9"\n')
        f.write(f'LABEL software.name = "{container_name}"\n')
        f.write('RUN apt-get -y update && apt-get -y install python3-pip zip unzip\n')
        f.write('RUN pip install awscli boto3\n')
        # install a few core libraries
        f.write('RUN conda install -c bioconda bedtools\n')
        f.write('RUN conda install -c bioconda bcftools\n')
        # f.write('RUN conda install -c bioconda bedops\n')
        # package installs from requirements
        for p in pkg2install:
            f.write(f'RUN {p}\n')
        # extra docker commands to run
        for c in cmd2run:
            dclist = translate_command_to_docker(dict(command=c, name=container_name))['docker_commands']
            for dc in dclist:
                f.write(dc+'\n')
        # samtools
        f.write('RUN mkdir /samtools\n')
        f.write('ENV PATH="/samtools/:$PATH"\n')
        f.write('COPY samtools/bin/* /samtools/\n')
        # copy scripts
        f.write('WORKDIR /\n')
        for u in utils_files:
            f.write(f'COPY {u} /\n')
        f.write('COPY run_main.py /\n')
        f.write('ENV PATH="/usr/local/bin/:$PATH"\n')
        f.write('ENTRYPOINT ["python", "/run_main.py"]\n')
    return dict(dockerfile=dockerfile_name)

def write_dockerfile_R( args ):
    """
    dockerfile: name of dockerfile
    ---
    dockerfile: name of dockerfile
    """
    dockerfile_name = args["dockerfile"]
    container_name = args["name"]
    pkg2install = args['packages']
    cmd2run = args['commands']
    utils_files = args['utils']

    with open(dockerfile_name,'w') as f:
        f.write("FROM rocker/tidyverse:4.1.0\n")
        f.write('LABEL container.base.image = "rocker:tidyverse"\n')
        f.write(f'LABEL software.name = "{container_name}"\n')
        f.write('RUN apt-get -y update && apt-get -y install python3-pip zip unzip\n')
        f.write('RUN pip install awscli boto3\n')
        # install a few core libraries
        f.write('RUN echo \'local({r <- getOption("repos"); r["CRAN"] <- "http://cran.r-project.org"; options(repos=r)})\' > /.Rprofile\n')
        f.write('RUN ln -s /usr/local/lib/R/lib/libR.so /lib/x86_64-linux-gnu/libR.so\n')
        f.write('RUN R -e \'install.packages("png")\'\n')
        # package installs from requirements
        for p in pkg2install:
            if 'install.packages' in p:
                f.write(f"RUN R -e '{p}'\n")
            else:
                f.write(f"RUN {p}\n")
        # extra docker commands to run
        for c in cmd2run:
            dclist = translate_command_to_docker(dict(command=c))['docker_commands']
            for dc in dclist:
                f.write(dc+'\n')
        # samtools
        f.write('RUN mkdir /samtools\n')
        f.write('ENV PATH="/samtools/:$PATH"\n')
        f.write('COPY samtools/bin/* /samtools/\n')
        # copy scripts
        f.write('WORKDIR /\n')
        for u in utils_files:
            f.write(f'COPY {u} /\n')
        f.write('COPY run_main.py /\n')
        f.write('ENV PATH="/usr/local/bin/:$PATH"\n')
        f.write('ENTRYPOINT ["python3", "/run_main.py"]\n')
    return dict(dockerfile=dockerfile_name)



import os, sys, subprocess, json

SCRIPT_DIR = str(os.path.dirname(os.path.realpath(__file__)))
HOME_PATH = os.path.expanduser('~')
INIT_PATH = os.path.join(HOME_PATH, '.bioshedinit/')
VALID_REGIONS = ['us-west-2', 'us-west-1', 'us-east-1', 'us-east-2', 'af-south-1', 'ap-east-1', \
                 'ap-southeast-3', 'ap-south-1', 'ap-northeast-3', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2', 'ap-northeast-1', \
                 'ca-central-1', 'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-south-1', 'eu-west-3', 'eu-north-1', \
                 'me-south-1', 'me-central-1', 'se-east-1']
ECR_PUBLIC_REGISTRY = "public.ecr.aws/w7q0j5w1"
BIOSHED_SERVERLESS_API = "https://hu9ug76w32.execute-api.us-west-2.amazonaws.com/prod"

sys.path.append(os.path.join(SCRIPT_DIR, 'bioshed_utils/'))
import quick_utils

def userExists( user ):
    """ Check if username exists in the user database.
    """
    mybody = {"user": user}
    user_exists = quick_utils.post_request(dict(url=BIOSHED_SERVERLESS_API+'/searchusers', body=mybody))
    if "user_found" in user_exists and str(user_exists["user_found"])[0].upper() == "T":
        return True
    else:
        return False

def bioshed_login():
    """ Log into BioShed.
    """
    myuser = input('Enter your BioShed username (email address): ') or ""
    user_exists = False
    if myuser != "":
        user_exists = userExists(myuser)
    if not user_exists:
        print("ERROR: User {} does not exist.".format(str(myuser)))
    return {"login": False, "user": myuser} if not user_exists else {"login": True, "user": myuser}

def bioshed_init( args ):
    """
    system: ubuntu, macosx,...
    cloud: aws, gcp,...
    initpath: path to all init and setup files
    configfile: config file for important config constants
    providerfile: TF provider file
    mainfile: TF main file
    ---
    provider_file

    """
    system_type = args['system']
    which_os = ''

    if 'ubuntu' in system_type.lower():
        bioshed_init_ubuntu()
        which_os = 'ubuntu'
    elif 'debian' in system_type.lower():
        bioshed_init_ubuntu()
        which_os = 'debian'
    elif 'redhat' in system_type.lower():
        bioshed_init_redhat()
        which_os = 'redhat'
    elif 'macosx' in system_type.lower():
        bioshed_init_macosx()
        which_os = 'macosx'
    elif 'amazon' in system_type.lower():
        bioshed_init_amazonlinux()
        which_os = 'amazonlinux'
    else:
        print('ERROR: Unsupported system OS - exiting.')

    return which_os

def bioshed_setup( args ):
    """
    cloud: aws, gcp,...
    initpath: path to all init and setup files
    configfile: config file for important config constants
    providerfile: TF provider file
    mainfile: TF main file
    ---
    provider_file

    [TODO] fix this: Found preexisting AWS CLI installation: /usr/local/aws-cli/v2/current. Please rerun install script with --update flag.
    """
    cloud_provider = args['cloud']
    init_path = args['initpath']
    config_file = args['configfile']
    provider_file = args['providerfile']
    main_file = args['mainfile']

    if cloud_provider.lower() in ['aws','amazon']:
        bioshed_init_aws()
        provider_file = bioshed_setup_aws( dict(initpath=init_path, configfile=config_file, providerfile=provider_file, mainfile=main_file))
    return provider_file

def bioshed_init_macosx():
    # first make sure brew is installed: https://brew.sh/
    # install pip
    # subprocess.call('brew install brew-pip', shell=True)
    # install terraform
    subprocess.call('brew tap hashicorp/tap', shell=True)
    subprocess.call('brew install hashicorp/tap/terraform', shell=True)
    return

def bioshed_init_amazonlinux():
    # install pip
    subprocess.call('sudo yum -y update', shell=True)
    subprocess.call('sudo yum -y install python3-pip zip unzip', shell=True)

    # install terraform: https://learn.hashicorp.com/tutorials/terraform/install-cli?in=terraform/aws-get-started
    subprocess.call('sudo yum install -y yum-utils', shell=True)
    subprocess.call('sudo yum-config-manager --add-repo https://rpm.releases.hashicorp.com/AmazonLinux/hashicorp.repo', shell=True)
    subprocess.call('sudo yum -y install terraform', shell=True)

    # make sure terraform works
    subprocess.call('terraform -help', shell=True)
    return

def bioshed_init_redhat():
    # install pip
    subprocess.call('sudo yum -y update', shell=True)
    subprocess.call('sudo yum -y install python3-pip zip unzip', shell=True)

    # install terraform: https://learn.hashicorp.com/tutorials/terraform/install-cli?in=terraform/aws-get-started
    subprocess.call('sudo yum install -y yum-utils', shell=True)
    subprocess.call('sudo yum-config-manager --add-repo https://rpm.releases.hashicorp.com/RHEL/hashicorp.repo', shell=True)
    subprocess.call('sudo yum -y install terraform', shell=True)

    # make sure terraform works
    subprocess.call('terraform -help', shell=True)
    return

def bioshed_init_ubuntu():
    # install pip
    subprocess.call('sudo apt-get -y update', shell=True)
    subprocess.call('sudo apt-get -y install python3-pip zip unzip', shell=True)

    # install terraform: https://learn.hashicorp.com/tutorials/terraform/install-cli?in=terraform/aws-get-started
    subprocess.call('sudo apt-get install -y gnupg software-properties-common', shell=True)
    subprocess.call('wget -O- https://apt.releases.hashicorp.com/gpg | gpg --dearmor | sudo tee /usr/share/keyrings/hashicorp-archive-keyring.gpg', shell=True)
    subprocess.call('gpg --no-default-keyring --keyring /usr/share/keyrings/hashicorp-archive-keyring.gpg --fingerprint', shell=True)
    subprocess.call('echo "deb [signed-by=/usr/share/keyrings/hashicorp-archive-keyring.gpg] https://apt.releases.hashicorp.com $(lsb_release -cs) main" | sudo tee /etc/apt/sources.list.d/hashicorp.list', shell=True)
    subprocess.call('sudo apt update', shell=True)
    subprocess.call('sudo apt-get install terraform', shell=True)

    # boto3 for python-based infra control
    subprocess.call('pip install boto3', shell=True)

    # make sure terraform works
    subprocess.call('terraform -help', shell=True)
    return

def bioshed_init_aws():
    # install aws cli and configure
    subprocess.call('pip install awscli', shell=True)
    subprocess.call('curl "https://awscli.amazonaws.com/awscli-exe-linux-x86_64.zip" -o "awscliv2.zip"', shell=True)
    subprocess.call('unzip awscliv2.zip', shell=True)
    subprocess.call('sudo ./aws/install', shell=True)
    subprocess.call('aws configure', shell=True)

def bioshed_setup_aws( args ):
    """
    initpath: init path
    configfile: config file for important config constants
    providerfile: TF provider file
    mainfile: TF main file
    ---
    """
    # setup terraform provider (AWS): https://learn.hashicorp.com/tutorials/terraform/aws-build
    cwd = os.getcwd()
    INIT_PATH = args['initpath']
    if not os.path.exists(INIT_PATH):
        os.mkdir(INIT_PATH)
    PROVIDER_FILE = args['providerfile'] # os.path.join(INIT_PATH, 'hs_providers.tf')
    MAIN_FILE = args['mainfile'] # os.path.join(INIT_PATH, 'main.tf')
    AWS_CONFIG_FILE = args['configfile']
    KEYS_FILE = os.path.join(INIT_PATH, 'hsconfig.tf')
    AWS_REGION = input('Which region do you want to setup your bioshed infrastructure? (ex: us-west-2, us-east-1, eu-south-1, ca-central-1): ')
    if AWS_REGION not in VALID_REGIONS:
        AWS_REGION = input('Please type a valid AWS region (default: us-west-2): ') or "us-west-2"
    AWS_CONSTANTS_JSON = {}
    AWS_CONSTANTS_JSON["ecr_registry"] = ECR_PUBLIC_REGISTRY
    AWS_CONSTANTS_JSON["aws_region"] = AWS_REGION

    PKEY_INPUT = input('(Needed for running stuff on the cloud): Provide a valid public key for accessing AWS resources (in a separate window, type "ssh-keygen" and paste the public key here) or press ENTER to skip for now: ')
    with open(PROVIDER_FILE,'w') as f:
        f.write('terraform {\n')
        f.write('  required_providers {\n')
        f.write('    aws = {\n')
        f.write('      source  = "hashicorp/aws"\n')
        f.write('      version = "~> 3.0"\n')
        f.write('    }\n')
        f.write('  }\n')
        f.write('}\n')
        f.write('\n')
        f.write('provider "aws" {\n')
        f.write('  region = "{}"\n'.format(AWS_REGION))
        f.write('  shared_credentials_file = "$HOME/.aws/credentials"\n')
        f.write('}\n')
    with open(MAIN_FILE,'w') as f:
        f.write('\n')
    if PKEY_INPUT not in ['',' ',[]]:
        with open(KEYS_FILE,'w') as f:
            f.write(
            """
            resource "aws_key_pair" "deployer" {
              key_name   = "bioshed-managed-key-for-ec2-instances"
            """
            )
            f.write('  public_key = "{}"\n'.format(PKEY_INPUT))
            f.write('}\n\n')
    with open(AWS_CONFIG_FILE,'w') as f:
        json.dump(AWS_CONSTANTS_JSON, f)
    os.chdir(INIT_PATH)
    # initialize terraform
    subprocess.call('terraform init', shell=True)
    os.chdir(cwd)
    return PROVIDER_FILE

def bioshed_teardown( args ):
    """ Destroys cloud infrastructure setup by bioshed.
    Be very careful before doing this.

    initpath: init path
    """
    cwd = os.getcwd()
    INIT_PATH = args['initpath']
    os.chdir(INIT_PATH)
    subprocess.call('terraform plan -destroy', shell=True)
    subprocess.call('terraform apply -destroy', shell=True)
    os.chdir(cwd)
    return



import os, sys, uuid, json, boto3
import quick_utils
from datetime import datetime

SCRIPT_DIR = str(os.path.dirname(os.path.realpath(__file__)))
HOME_PATH = os.path.expanduser('~')
AWS_CONFIG = quick_utils.loadJSON(os.path.join(HOME_PATH,'.bioshedinit/','aws_config_constants.json')) \
             if os.path.exists(os.path.join(HOME_PATH,'.bioshedinit/','aws_config_constants.json')) \
             else quick_utils.loadJSON(os.path.join(SCRIPT_DIR,'aws_config_constants.json'))
SPECS_CONFIG = quick_utils.loadJSON(os.path.join(SCRIPT_DIR, 'specs.json'))

def submit_job_awsbatch( args ):
    """
    name: name of container
    tag: (optional) version tag of container
    program_args: program arguments (as a string)
    dependent_job_ids: (optional) list of IDs of dependent jobs
    ---
    jobinfo: JSON of submitted job info

    >>> submit_job_awsbatch( dict(name='test', program_args=''))
    0
    """
    def setJobProperties( module_name, module_version ):
        job_properties = {}
        job_properties['image'] = f"{AWS_CONFIG['ecr_registry']}/{module_name}:{module_version}"
        job_properties['vcpus'] = int(SPECS_CONFIG[module_name]['vcpu']) if (module_name in SPECS_CONFIG and 'vcpu' in SPECS_CONFIG[module_name]) else 1
        job_properties['memory'] = int(SPECS_CONFIG[module_name]['mem']) if (module_name in SPECS_CONFIG and 'mem' in SPECS_CONFIG[module_name]) else 1000
        job_properties['jobRoleArn'] = AWS_CONFIG['aws_ecs_job_role']
        return job_properties

    def setContainerOverrides( program_args ):
        overrides = [program_args] if program_args != '' else ['-test']
        return overrides

    def createDependentIdList( jobid_list ):
        jobid_list_final=[]
        jobid_list = jobid_list.split(',') if type(jobid_list)==str else jobid_list
        for jobid in jobid_list:
            if jobid != '':
                jobid_list_final.append({'jobId': jobid})
        return jobid_list_final

    cname = args['name']
    pargs = quick_utils.format_type(args['program_args'], 'string_space') if 'program_args' in args else ''
    ctag = args['tag'] if 'tag' in args else 'latest'
    dependent_job_ids = args['dependent_job_ids'] if 'dependent_job_ids' in args else []
    jobinfo = {}

    # unique ID for this job
    uid = str(uuid.uuid4())

    # initialize Batch boto3 client access
    print('\nSetting up boto3 client in {}...'.format(AWS_CONFIG['aws_region']))
    client = boto3.client('batch', region_name=AWS_CONFIG['aws_region'])

    # set properties for this job
    job_properties = setJobProperties( cname, ctag )
    job_name = f'job_{cname}_{uid}'
    print('Setting job properties for job: '+str(job_name))

    # get dependent ids
    dependent_ids = createDependentIdList( dependent_job_ids )

    # get the date and time stamp right before we submit job
    job_submission_timestamp = str(datetime.now())

    # set input and compute parameters for job submission - save this job submission info
    job_json = {}
    job_overrides = {'command': setContainerOverrides(pargs)}
    job_json['container_overrides'] = job_overrides
    job_json['jobqueue'] = AWS_CONFIG['jobqueue']
    job_json['jobname'] = job_name
    job_json['job_submission_timestamp'] = job_submission_timestamp

    # register job definition
    job_def_name = f'jdef_{cname}_{uid}'
    job_def_response = client.register_job_definition( jobDefinitionName = job_def_name,
                                                       type='container',
                                                       retryStrategy={'attempts': 3},
                                                       containerProperties=job_properties)
    print('\nRegistering Job Definition: '+str(job_def_name))

    # submit job
    job_submit_response = client.submit_job( jobName = job_name,
                                             jobQueue = AWS_CONFIG['jobqueue'],
                                             jobDefinition = job_def_name,
                                             containerOverrides = job_overrides,
                                             dependsOn=dependent_ids)
    job_name_submitted = str(job_submit_response['jobName'])
    job_id_final = str(job_submit_response['jobId'])

    return dict(module=cname, jobid=job_id_final, jobqueue=AWS_CONFIG['jobqueue'],
                joboverrides=job_overrides, program_args=pargs)



import sys, os, subprocess, json
import aws_s3_utils
import quick_utils

def init_program( args ):
    """ Initialize program run.

    program_args: raw program arguments (as list)
    ---
    workingdir: working directory of container
    inputdir: input directory (where files are downloaded)
    outputdir: output directory (these files will be uploaded)
    remote_outputdir: remote output directory to eventually upload to
    program_args: formatted program arguments (list of commands)
       - // can be used to delimit multiple commands
       - e.g. cat hello.txt // echo hello

    [TODO] currently a hack-fix for checking pargs passed in - make cleaner
    """
    cwd = os.getcwd()
    inputdir = os.path.join(cwd,'hubinput/')
    outputdir = os.path.join(cwd,'huboutput/')
    os.mkdir(inputdir)
    os.mkdir(outputdir)
    pargs_passed_in = args['program_args']
    if type(pargs_passed_in)==type([]) and len(pargs_passed_in)==1:
        # program arguments passed in as a single-entry list (by Batch)
        pargs_passed_in = pargs_passed_in[0]
        pargs = quick_utils.args_to_list( pargs_passed_in, ' ')
    elif type(pargs_passed_in)==type([]) and len(pargs_passed_in)>1:
        # arguments passed in as a full list:
        pargs = pargs_passed_in
    else:
        # arguments passed in as a space-delimited string
        pargs = quick_utils.args_to_list( pargs_passed_in, ' ')

    pargs_all = []
    pargs_formatted = []
    for parg in pargs:
        if not parg.startswith('out::') and not parg.startswith('arg://') and parg.strip(' ')!='//':
            pargs_formatted.append(parg)
        elif parg.strip(' ')=='//' and pargs_formatted != []:
            # user passed multiple commands
            pargs_all.append(' '.join(pargs_formatted) )
            pargs_formatted = []
    if pargs_formatted != []:
        pargs_all.append( ' '.join(pargs_formatted) )

    remote_outputdir = get_remote_outputdir( dict(program_args=pargs))
    alt_inputs = get_alternate_inputs( dict(program_args=pargs, localdir=inputdir))

    params = {}
    params['workingdir'] = cwd.rstrip('/')+'/'
    params['inputdir'] = inputdir
    params['outputdir'] = outputdir
    params['remote_outputdir'] = remote_outputdir
    params['program_args'] = pargs_all
    params['alt_inputs'] = alt_inputs
    return params

def get_remote_outputdir( args ):
    """ Given program arguments, gets explicit remote output directory (or infers)

    program_args: program arguments
    cloud: remote cloud prefix (s3:// gcp:// )
    ---
    remote_outputdir: remote output directory
    """
    pargs = quick_utils.args_to_list( args['program_args'], ' ')
    cloud_prefix = args['cloud'] if 'cloud' in args else 's3://'
    remote_alt = ''
    remote_explicit = ''
    for parg in pargs:
        if parg.startswith(cloud_prefix):
            remote_alt = quick_utils.get_file_folder(parg)
        elif parg.startswith('out::'):
            remote_explicit = parg[5:]
    return remote_explicit if remote_explicit!='' else remote_alt

def get_alternate_inputs( args ):
    """ Given program arguments, downloads alternate input files if provided

    program_args: program arguments
    localdir: local directory to download files to
    ---
    alt_inputs: space-separated string of alternate input files
    """
    pargs = quick_utils.args_to_list( args['program_args'], ' ')
    localdir = args['localdir'] if 'localdir' in args else '/hubinput/'
    alt_files = ''
    for parg in pargs:
        if parg.startswith('alt::'):
            alt_files += parg[5:]+' '
    alt_files_local = download_files( dict(program_args=alt_files.strip(), localdir=localdir))
    return alt_files_local


def download_files( args ):
    """ Given program arguments, downloads any cloud files.
    program_args: program arguments (as a list of space-separated string of args)
    localdir: directory to download to
    ---
    program_args_out: formatted program arguments

    >>> download_files( dict(program_args=['cat s3://hubscratch/test-policies.txt', 'echo hello'], localdir='./'))

    """
    localdir = args['localdir']
    pargs_list = args['program_args']

    # a list of commands is passed in as program_args, so we iterate through this list
    program_args_out_all = []
    for pargs_command in pargs_list:
        pargs = quick_utils.args_to_list( pargs_command, ' ')
        program_args_out = []
        for p in pargs:
            # download from AWS
            if p.startswith('s3://') and aws_s3_utils.object_exists_s3( dict(path=p)):
                # download a file
                if '.' in p.split('/')[-1] and not p.endswith('/'):
                    out = aws_s3_utils.download_file_s3( dict(path=p, localdir=localdir))
                # download a folder
                else:
                    out = aws_s3_utils.download_folder_s3( dict(path=p, localdir=localdir))
                program_args_out.append(out)
            # download from GCP
            elif p.startswith('gcp://'):
                # placeholder for now
                program_args_out.append(p)
            else:
                program_args_out.append(p)
        if program_args_out != []:
            program_args_out_all.append(' '.join(program_args_out))

    return program_args_out_all


def run_program( args ):
    """ Runs program on command line.
    command: full command with program args to run (string)
    logfile: run log file name
    ---

    [TODO] try except for subprocess
    """
    try:
        cmd = args['command']
        logfile = args['logfile']

        subprocess.call(cmd+' > {}'.format(logfile), shell=True)
        if len(cmd.split(' ')) <= 1:
            subprocess.call(cmd+' --help', shell=True)
        return
    except:
        print('Error in running program: '+str(cmd))
        return

def log_run( args ):
    """ Logs program run with other information.
    command: full command with program args (string)
    program_logfile: name of program output log file
    run_logfile: name of entire run log file (contains additional metadata for this run)
    run_duration: ...
    ---

    """
    cmd = args['command']
    program_logfile = args['program_logfile']
    run_logfile = args['run_logfile']
    run_duration = args['run_duration']

    out = {'command': cmd, 'program_logfile': program_logfile,
           'run_logfile': run_logfile, 'run_duration': run_duration}
    quick_utils.writeJSON( out, run_logfile )
    return

def upload_output( args ):
    """ Upload all files in output directory to remote cloud

    local_outputdir: local output directory
    remote_outputdir: remote output directory
    ---
    remote_outputdir: ...
    """
    out = aws_s3_utils.upload_folder_s3( dict(localfolder=args['local_outputdir'], path=args['remote_outputdir']))
    return dict(remote_outputdir=out)



import os, sys, json
SCRIPT_DIR = str(os.path.dirname(os.path.realpath(__file__)))
HOME_PATH = os.path.expanduser('~')
INIT_PATH = os.path.join(HOME_PATH, '.bioshedinit/')

import bioshed_core_utils
import bioshed_init
import bioshed_deploy_core
sys.path.append(os.path.join(SCRIPT_DIR, 'bioshed_utils/'))
import docker_utils
import aws_batch_utils
import quick_utils
sys.path.append(os.path.join(SCRIPT_DIR, 'bioshed_atlas/'))
import atlas_encode_utils

AWS_CONFIG_FILE = os.path.join(INIT_PATH,'aws_config_constants.json')
PROVIDER_FILE = os.path.join(INIT_PATH, 'hs_providers.tf')
MAIN_FILE = os.path.join(INIT_PATH, 'main.tf')
SYSTEM_TYPE = bioshed_core_utils.detect_os()
VALID_COMMANDS = ['init', 'setup', 'build', 'run', 'runlocal', 'deploy', 'search', 'download', 'teardown']

def bioshed_cli_entrypoint():
    bioshed_cli_main( sys.argv )
    return

def bioshed_cli_main( args ):
    """ Main function for parsing command line arguments and running stuff.
    args: list of command-line args

    [TODO] figure out local search
    """
    if SYSTEM_TYPE == 'unsupported' or SYSTEM_TYPE == 'windows': # until I can support windows
        print('Unsupported system OS. Linux (Ubuntu, Debian, RedHat, AmazonLinux) or Mac OS X currently supported.\n')
    elif len(args) > 1:
        cmd = args[1].strip()
        if cmd in ['run', 'runlocal'] and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            if len(args) < 3:
                print('You must specify a module with at least one argument - ex: bioshed run fastqc -h')
                return
            args = args[2:] # don't need to parse "bioshed run/runlocal"
            dockerargs = ''
            # optional argument is specified
            while args[0].startswith('--'):
                if args[0]=='--aws-env-file':
                    if len(args) < 2:
                        print('Either did not specify env file or module name - ex: bioshed runlocal --aws-env-file .env fastqc -h')
                    dockerargs = '--env-file {}'.format(args[1])
                    args = args[2:]
                elif args[0]=='--local':
                    cmd = 'runlocal'
                    args = args[1:]
            module = args[0].strip()
            # run module
            if cmd == 'run':
                jobinfo = aws_batch_utils.submit_job_awsbatch( dict(name=module, program_args=args))
                print('SUBMITTED JOB INFO: '+str(jobinfo))
            elif cmd == 'runlocal':
                print('NOTE: If you get an AWS credentials error, you may need to specify an AWS ENV file: --aws-env-file <.ENV>')
                docker_utils.run_container_local( dict(name=module, args=args, dockerargs=dockerargs))

        elif cmd == 'build' and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            if len(args) < 3:
                print('You must specify a module to build: bioshed build <MODULE> <ARGS>')
                return
            module = args[2].strip()
            parsed_args = bioshed_core_utils.parse_build_args( args[3:] )
            print('MODULE: '+str(module))
            if 'install' in parsed_args:
                docker_utils.build_container( dict(name=module, requirements=parsed_args.install, codebase=parsed_args.codebase ))
        elif cmd == 'setup' and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            if len(args) > 2:
                cloud_provider = args[2].lower()
                if cloud_provider in ['aws', 'amazon']:
                    bioshed_init.bioshed_setup(dict( cloud=cloud_provider, initpath=INIT_PATH, configfile=AWS_CONFIG_FILE, providerfile=PROVIDER_FILE, mainfile=MAIN_FILE))
                else:
                    print('Provider {} currently not supported.'.format(cloud_provider))
            else:
                print('Must specify a cloud provider - e.g., bioshed setup aws')
        elif cmd == 'init':
            # ask for login name (email)
            login_success = bioshed_init.bioshed_login()
            if login_success["login"]:
                quick_utils.add_to_json(AWS_CONFIG_FILE, {"login": login_success["user"]})
                which_os = bioshed_init.bioshed_init(dict(system=SYSTEM_TYPE))
                print('BioShed initial install complete. Follow-up options are:')
                print('1) Type "bioshed setup aws" and then "bioshed deploy core" to setup AWS infrastructure for Bioshed.')
                print('2) Type "bioshed build <module> <args>" to build a new bioinformatics application module.')
                print('3) Type "bioshed search encode/ncbi/local/etc..." to search a system or repository for datasets.')
                print('')
        elif cmd == 'deploy' and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            if len(args) < 3:
                print('Must specify a resource to deploy - e.g., bioshed deploy core\n')
                return
            deploy_resource = args[2]
            # for now, assume cloud provider is AWS
            provider = 'aws'
            deploy_option = args[3] if len(args) > 3 else ''
            bioshed_deploy_core.bioshed_deploy_core(dict(cloud_provider=provider, initpath=INIT_PATH, configfile=AWS_CONFIG_FILE, deployoption=deploy_option))
        elif cmd == 'teardown' and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            r = input('You are going to tear down your entire bioshed infrastructure. Are you sure? y/n: ') or "N"
            # have another credential-based check - ask for AWS credentials or some password
            if r.upper() == "Y":
                bioshed_init.bioshed_teardown( dict(initpath=INIT_PATH))
        elif cmd == 'search' and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            if len(args) < 3:
                print('Specify a system or repository to search and type your search terms. Examples:')
                print('\tbioshed search encode <SEARCH_TERMS>')
                print('\tbioshed search ncbi <SEARCH_TERMS>')
                print('\tbioshed search local <SEARCH_TERMS>')
                return
            if str(args[2]).lower() == 'encode':
                search_terms = str(' '.join(args[3:])).strip()
                print('Searching ENCODE for: {}'.format(search_terms))
                atlas_encode_utils.search_encode( dict(searchterms=search_terms))
            elif str(args[2]).lower() == 'ncbi':
                print('NCBI search coming soon!')
            elif str(args[2]).lower() == 'local':
                print('Local search coming soon!')
            else:
                print('Currently supported searches: encode, nbci, local')
        elif cmd == 'download' and bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            if len(args) < 3:
                print('Specify a system or repository to download files from. Examples:')
                print('\tbioshed download encode')
                print('\tbioshed download ncbi')
                print('\tbioshed download local')
                return
            if str(args[2]).lower() == 'encode':
                atlas_encode_utils.download_encode( dict(downloadstr=str(' '.join(args[3:])).strip()))
            elif str(args[2]).lower() == 'ncbi':
                print('NCBI download coming soon!')
            elif str(args[2]).lower() == 'local':
                print('Local download coming soon!')
            else:
                print('Currently supported downloads: encode, nbci, local')
        elif cmd in VALID_COMMANDS and not bioshed_init.userExists( dict(quick_utils.loadJSON(AWS_CONFIG_FILE)).get("login", "") ):
            print('Not logged on. Please type "bioshed init" and login first.')
        else:
            print_help_menu()
    else:
        print_help_menu()
    return

def print_help_menu():
    print('Specify a valid subcommand. Valid subcommands are:\n')
    print('\t$ bioshed init')
    print('\t$ bioshed setup aws')
    print('\t$ bioshed deploy core')
    print('\t$ bioshed teardown aws')
    print('')
    print('\t$ bioshed run')
    print('\t$ bioshed build')
    print('')
    print('\t$ bioshed search encode')
    print('\t$ bioshed search ncbi')
    print('\t$ bioshed search local')
    print('')
    print('\t$ bioshed download encode')
    print('\t$ bioshed download ncbi')
    print('\t$ bioshed download local')
    print('')
    return



import os, subprocess, sys
from datetime import datetime
import program_utils
import file_utils

def run_main( pargs ):
    # time the duration of module container run
    run_start = datetime.now()
    run_start_string = str(run_start).replace(' ','-').replace(':','-').split('.')[0]
    print('Container running...')
    print('Program arguments: {}'.format(str(pargs)))

    # initialize program run - create input and output directories, format program arguments, etc
    params = program_utils.init_program( dict(program_args=pargs))
    
    # download any cloud files
    pargs_list_updated = program_utils.download_files( dict(program_args=params['program_args'], localdir=params['inputdir']))
    print('Program arguments after download: {}'.format(str(pargs_list_updated)))
    
    # run main program - run within output directory
    os.chdir(params['outputdir'])
    program_log_name_list = []
    cmd_order = 1
    for pargs_updated in pargs_list_updated:
        program_log_name = os.path.join(params['outputdir'], 'program.{}.{}.{}.log'.format(str(pargs_updated.split(' ')[0]), run_start_string, str(cmd_order)))
        program_utils.run_program( dict(command=pargs_updated, logfile=program_log_name ))
        program_log_name_list.append(program_log_name)
        cmd_order += 1
    
    # create run log that includes program run duration
    run_end = datetime.now()
    run_log_name = os.path.join(params['outputdir'], 'run.{}.log'.format(str(run_start_string)))
    program_utils.log_run( dict(command=pargs_list_updated, program_logfile=program_log_name_list,
                                run_logfile=run_log_name,
                                run_duration=str(run_end - run_start)))

    # upload output data files
    program_utils.upload_output( dict(local_outputdir=params['outputdir'], remote_outputdir=params['remote_outputdir']))
    print('Container finished in {}'.format(str(run_end - run_start)))
    return


if __name__ == '__main__':
    run_main(sys.argv[1:]) if len(sys.argv) > 1 else run_main([])



from setuptools import setup
setup(
    name='bioshed',
    version='0.1.8',
    description='BioShed Cloud Bioinformatics Tookit',
    install_requires=[
        'boto3',
        'pyyaml'
    ],
    python_requires='>=3.0',            # Minimum version requirement of the package
    py_modules=["bioshed"],             # Name of the python package
    package_dir={'bioshed':'bioshed/src'},     # Directory of the source code of the package
    packages=['bioshed/src/', 'bioshed/src/bioshed_utils', 'bioshed/src/bioshed_atlas', 'bioshed/src/bioshed_atlas/bioshed_utils', 'bioshed/src/bioshed_atlas/files'],
    package_data={'bioshed.src.bioshed_utils': ['bioshed_utils/*.json', 'bioshed_utils/*.yaml'], \
                  'bioshed.src.bioshed_atlas.files': ['bioshed_atlas/files/*.txt']},
    entry_points={
        'console_scripts': [
            'bioshed=bioshed.src.bioshed:bioshed_cli_entrypoint'
        ]
    }
)



import os

def getParameter( param_dict, k, v_default ):
    """ Return value of key k in param_dict, if found - otherwise return v_default.
    """
    if k in param_dict:
        return param_dict[k]
    else:
        return v_default

def getS3path( partialFilePaths, teamid = '', userid = '', useBaseDir = 'false' ):
    """ Given a list of partial input file paths (comma-separated string or list),
        prepends the S3 bucket name.
        Return full file paths in the same type provided as input.

        useBaseDir - whether or not to use the basedir on an empty string
    
    >>> lambda_utils.getS3path('bioshed/test/file1.pdf')
    's3://biousers/bioshed/test/file1.pdf'
    >>> lambda_utils.getS3path('bioshed/test/file1.pdf,bioshed/file2.txt')
    's3://biousers/bioshed/test/file1.pdf,s3://biousers/bioshed/file2.txt'
    >>> lambda_utils.getS3path(['bioshed/test/file1.pdf'])
    ['s3://biousers/bioshed/test/file1.pdf']
    >>> lambda_utils.getS3path(['bioshed/test/file1.pdf', 'bioshed/file2.txt'])
    ['s3://biousers/bioshed/test/file1.pdf', 's3://biousers/bioshed/file2.txt']
    >>> lambda_utils.getS3path(['s3://bioshed-data/test/file1.pdf', 's3://bioshed-data/test/file2.txt'])
    ['s3://bioshed-data/test/file1.pdf', 's3://bioshed-data/test/file2.txt']
    """
    TEAM_BUCKET = 's3://biousers/'
    if teamid != '':
        TEAM_BUCKET += teamid+'/'
    # if userid != '':
    #    TEAM_BUCKET += userid+'/'

    # create list of partial file paths from input
    if type(partialFilePaths)==type(''):
        partialFilePathsList = partialFilePaths.lstrip("'").lstrip('"').rstrip("'").rstrip('"').split(',')
        returnType = "string"
    elif  type(partialFilePaths)==type([]):
        partialFilePathsList = partialFilePaths
        returnType = "list"
    else:
        partialFilePathsList = []
        returnType = "list"

    # create full filepaths
    fullPaths = []
    for f in partialFilePathsList:
        if ('s3://' not in f and f not in ['~/','',"''",'""']) or (f in ['~/','',"''",'""'] and useBaseDir.lower() != 'false'):
            fullPaths.append(os.path.join(TEAM_BUCKET, f.lstrip('/')))
        elif useBaseDir.lower() == 'false' and f in ['',"''",'""']:
            fullPaths.append("''")
        else:
            fullPaths.append(f)

    # format and return full filepaths
    if returnType=="string":
        return ','.join(fullPaths)
    else:
        return fullPaths
    

def getS3path_args( argslist_input, teamid = '', userid = '' ):
    """ Takes an arguments list (either space-delimited string or a list) and
        for any paths that are marked with ~/
        replaces with S3 TEAM_BUCKET name.

        Special case if we have a string of strings (module_args in run_pipeline)
         - just replace ~/ with S3 TEAM BUCKET name.

        useBaseDir - whether or not to use the basedir on an empty string
    
    >>> lambda_utils.getS3path_args(['-a','-b','~/test/temp.txt','/blah'])
    ['-a', '-b', 's3://biousers/test/temp.txt', '/blah']
    >>> lambda_utils.getS3path_args('-a -b ~/test/temp.txt /blah')
    '-a -b s3://biousers/test/temp.txt /blah'
    """
    TEAM_BUCKET = 's3://biousers/'
    if teamid != '':
        TEAM_BUCKET += teamid+'/'
    if userid != '':
        TEAM_BUCKET += userid+'/'
    
    # create list of partial file paths from input
    if type(argslist_input)==type(''):
        return argslist_input.replace('~/',TEAM_BUCKET)
    elif type(argslist_input)==type([]):
        argsList = argslist_input
        # create full filepaths
        fullArgsList = []
        for f in argsList:
            if f.startswith('~/'):
                fullArgsList.append(os.path.join(TEAM_BUCKET, f[2:]))
            else:
                fullArgsList.append(f)
        return fullArgsList
    else:
        return []
    
